@@ -42,29 +42,24 @@ BoolSettings = Literal[
     'ask_online_status',
     'autoaway',
     'autoxa',
-    'change_roster_title',
     'chat_merge_consecutive_nickname',
     'check_for_update',
-    'command_system_execute',
     'confirm_close_muc',
-    'confirm_close_multiple_tabs',
     'confirm_on_window_delete',
-    'confirm_paste_image',
     'dev_force_bookmark_2',
     'dev_use_message_label',
     'developer_modus',
+    'enable_emoji_shortcodes',
     'enable_negative_priority',
+    'enable_file_preview',
     'escape_key_closes',
     'gc_notify_on_all_messages_private_default',
     'gc_notify_on_all_messages_public_default',
     'gc_print_join_left_default',
     'gc_print_status_default',
-    'hide_chat_banner',
-    'hide_groupchat_banner',
     'hide_groupchat_occupants_list',
     'ignore_incoming_attention',
     'is_window_visible',
-    'log_contact_status_changes',
     'muc_prefer_direct_msg',
     'notification_preview_message',
     'notify_on_all_muc_messages',
@@ -81,7 +76,6 @@ BoolSettings = Literal[
     'remote_control',
     'save_main_window_position',
     'send_on_ctrl_enter',
-    'shell_like_completion',
     'show_chatstate_in_banner',
     'show_help_start_chat',
     'show_in_taskbar',
@@ -107,7 +101,6 @@ BoolSettings = Literal[
     'use_stun_server',
     'use_urgency_hint',
     'video_see_self',
-    'enable_emoji_shortcodes',
 ]
 
 IntSettings = Literal[
@@ -121,7 +114,6 @@ IntSettings = Literal[
     'gc_sync_threshold_private_default',
     'gc_sync_threshold_public_default',
     'groupchat_roster_width',
-    'key_up_lines',
     'mainwin_height',
     'mainwin_width',
     'mainwin_x_position',
@@ -131,7 +123,6 @@ IntSettings = Literal[
     'notification_timeout',
     'preview_max_file_size',
     'preview_size',
-    'restore_timeout',
 ]
 
 StringSettings = Literal[
@@ -168,7 +159,7 @@ StringSettings = Literal[
 ]
 
 AllSettings = Literal[BoolSettings, IntSettings, StringSettings]
-AllSettingsT = Union[str, int, bool]
+AllSettingsT = Union[str, int, bool, list[str]]
 
 APP_SETTINGS = {
     'always_english_wikipedia': False,
@@ -185,17 +176,13 @@ APP_SETTINGS = {
     'autoxa': True,
     'autoxa_message': '',
     'autoxatime': 15,
-    'change_roster_title': True,
     'chat_handle_position': 350,
     'chat_merge_consecutive_nickname': True,
     'chat_timestamp_format': '%H:%M',
     'check_for_update': True,
-    'command_system_execute': False,
     'confirm_block': '',
     'confirm_close_muc': True,
-    'confirm_close_multiple_tabs': True,
     'confirm_on_window_delete': True,
-    'confirm_paste_image': True,
     'dark_theme': 2,
     'date_timestamp_format': '%x',
     'dev_force_bookmark_2': False,
@@ -203,7 +190,9 @@ APP_SETTINGS = {
     'developer_modus': False,
     'dictionary_url': 'WIKTIONARY',
     'emoticons_theme': 'noto-emoticons',
+    'enable_emoji_shortcodes': True,
     'enable_negative_priority': False,
+    'enable_file_preview': True,
     'escape_key_closes': False,
     'file_transfers_port': 28011,
     'ft_add_hosts_to_send': '',
@@ -216,18 +205,14 @@ APP_SETTINGS = {
     'gc_sync_threshold_public_default': 1,
     'global_proxy': '',
     'groupchat_roster_width': 210,
-    'hide_chat_banner': False,
-    'hide_groupchat_banner': False,
     'hide_groupchat_occupants_list': False,
     'ignore_incoming_attention': False,
-    'key_up_lines': 25,
     'is_window_visible': True,
     'last_save_dir': '',
     'last_send_dir': '',
     'last_sounds_dir': '',
     'last_update_check': '',
     'latest_disco_addresses': '',
-    'log_contact_status_changes': False,
     'mainwin_height': 500,
     'mainwin_width': 1000,
     'mainwin_x_position': 0,
@@ -257,12 +242,10 @@ APP_SETTINGS = {
     'preview_verify_https': True,
     'print_status_in_chats': False,
     'remote_control': False,
-    'restore_timeout': -1,
     'roster_theme': 'default',
     'save_main_window_position': True,
     'search_engine': 'https://duckduckgo.com/?q=%s',
     'send_on_ctrl_enter': False,
-    'shell_like_completion': False,
     'show_chatstate_in_banner': True,
     'show_help_start_chat': True,
     'show_in_taskbar': True,
@@ -295,7 +278,6 @@ APP_SETTINGS = {
     'video_see_self': True,
     'video_size': '',
     'workspace_order': [],
-    'enable_emoji_shortcodes': True,
 }
 
 BoolAccountSettings = Literal[
@@ -339,6 +321,7 @@ StringAccountSettings = Literal[
     'client_cert',
     'custom_host',
     'custom_type',
+    'default_workspace',
     'file_transfer_proxies',
     'filetransfer_preference',
     'gc_send_chatstate_default',
@@ -351,7 +334,6 @@ StringAccountSettings = Literal[
     'name',
     'password',
     'proxy',
-    'recent_groupchats',
     'resource',
     'roster_version',
     'send_chatstate_default',
@@ -383,13 +365,13 @@ BoolGroupChatSettings = Literal[
     'notify_on_all_messages',
     'print_join_left',
     'print_status',
-    'send_chatstate',
     'send_marker',
 ]
 
 StringGroupChatSettings = Literal[
     'encryption',
     'speller_language',
+    'send_chatstate',
 ]
 
 IntGroupChatSettings = Literal[
@@ -403,17 +385,34 @@ AllGroupChatSettings = Literal[BoolGroupChatSettings,
 
 AllGroupChatSettingsT = Union[str, int, bool]
 
+
+BoolContactSettings = Literal[
+    'send_marker',
+]
+
+StringContactSettings = Literal[
+    'encryption',
+    'speller_language',
+    'send_chatstate',
+]
+
+AllContactSettings = Literal[BoolContactSettings,
+                             StringContactSettings]
+
+AllContactSettingsT = Union[str, bool]
+
+
 ACCOUNT_SETTINGS = {
     'account': {
         'account_color': 'rgb(85, 85, 85)',
         'account_label': '',
-        'active': True,
+        'active': False,
         'adjust_priority_with_status': False,
         'anonymous_auth': False,
         'answer_receipts': True,
         'attached_gpg_keys': '',
         'autoauth': False,
-        'autoconnect': False,
+        'autoconnect': True,
         'autopriority_away': 40,
         'autopriority_chat': 50,
         'autopriority_dnd': 20,
@@ -427,6 +426,7 @@ ACCOUNT_SETTINGS = {
         'custom_host': '',
         'custom_port': 5222,
         'custom_type': 'START TLS',
+        'default_workspace': '',
         'enable_gssapi': False,
         'enable_security_labels': False,
         'file_transfer_proxies': '',
@@ -449,19 +449,18 @@ ACCOUNT_SETTINGS = {
         'proxy': '',
         'publish_location': False,
         'publish_tune': False,
-        'recent_groupchats': '',
         'request_user_data': True,
         'resource': 'gajim.$rand',
         'restore_last_status': False,
         'roster_version': '',
-        'savepass': False,
+        'savepass': True,
         'send_chatstate_default': 'composing_only',
         'send_idle_time': True,
         'send_marker_default': True,
         'send_os_info': True,
         'send_time_info': True,
         'subscription_request_msg': '',
-        'sync_with_global_status': False,
+        'sync_with_global_status': True,
         'test_ft_proxies_on_startup': False,
         'use_custom_host': False,
         'use_ft_proxies': False,
@@ -497,24 +496,33 @@ StringWorkspaceSettings = Literal[
     'name',
 ]
 
-AllWorkspaceSettings = Literal[StringWorkspaceSettings, 'open_chats']
+AllWorkspaceSettings = Literal[StringWorkspaceSettings, 'chats']
 
-OpenChatSettingT = list[tuple[str, JID, str, bool]]
-AllWorkspaceSettingsT = Union[str, OpenChatSettingT]
+
+class OpenChatSettingDetails(TypedDict):
+    account: str
+    jid: JID
+    type: str
+    pinned: bool
+    position: int
+
+
+OpenChatsSettingT = list[OpenChatSettingDetails]
+AllWorkspaceSettingsT = Union[str, OpenChatsSettingT]
 
 
 class WorkspaceSettings(TypedDict):
     name: str
     color: str
     avatar_sha: str
-    open_chats: OpenChatSettingT
+    chats: OpenChatsSettingT
 
 
 WORKSPACE_SETTINGS: WorkspaceSettings = {
     'name': _('My Workspace'),
     'color': '',
     'avatar_sha': '',
-    'open_chats': [],
+    'chats': [],
 }
 
 
@@ -620,40 +628,30 @@ ADVANCED_SETTINGS = {
         'always_english_wiktionary': '',
         'ascii_formatting': _(
             'Treat * / _ pairs as possible formatting characters.'),
-        'change_roster_title': _(
-            'If enabled, Gajim will add * and [n] in contact list '
-            'window title.'),
         'chat_merge_consecutive_nickname': _(
             'Show message meta data (avatar, nickname, timestamp) only once, '
             'if there are multiple messages from the same sender within a '
             'specific timespan.'),
         'chat_timestamp_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
-        'command_system_execute': _(
-            'If enabled, Gajim will execute commands '
-            '(/show, /sh, /execute, /exec).'),
         'confirm_block': _(
             'Show a confirmation dialog to block a contact? Empty string '
             'means never show the dialog.'),
         'confirm_close_muc': _('Ask before closing a group chat tab/window.'),
-        'confirm_close_multiple_tabs': _(
-            'Ask before closing tabbed chat window if there are chats that '
-            'can lose data (chat, private chat).'),
         'confirm_on_window_delete': _(
             'Ask before quitting when Gajim’s window is closed'),
-        'confirm_paste_image': _('Ask before pasting an image.'),
         'date_timestamp_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
         'dev_force_bookmark_2': _('Force Bookmark 2 usage'),
         'dev_use_message_label': '',
         'developer_modus': '',
         'dictionary_url': _(
             'Either a custom URL with %%s in it (where %%s is the word/phrase)'
-            ' or \'WIKTIONARY\' (which means use Wikitionary).'),
+            ' or "WIKTIONARY" (which means use Wikitionary).'),
         'enable_negative_priority': _(
             'If enabled, you will be able to set a negative priority to your '
             'account in the Accounts window. BE CAREFUL, when you are logged '
             'in with a negative priority, you will NOT receive any message '
             'from your server.'),
-        'escape_key_closes': _('If enabled, pressing Esc closes a tab/window.'),
+        'escape_key_closes': _('Close a chat by pressing the Escape key'),
         'file_transfers_port': '',
         'ft_add_hosts_to_send': _(
             'List of send hosts (comma separated) in '
@@ -665,42 +663,30 @@ ADVANCED_SETTINGS = {
             'Character to add after nickname when using nickname completion '
             '(tab) in group chat.'),
         'groupchat_roster_width': _('Width of group chat roster in pixel'),
-        'hide_chat_banner': _('Hides the banner in a 1:1 chat window.'),
-        'hide_groupchat_banner': _('Hides the banner in a group chat window.'),
-        'hide_groupchat_occupants_list': _(
-            'Hides the group chat participants list in a group chat window.'),
         'ignore_incoming_attention': _(
             'If enabled, Gajim will ignore incoming attention '
-            'requests (\'wizz\').'),
-        'key_up_lines': _(
-            'How many lines to store for Ctrl+KeyUP '
-            '(previously sent messages).'),
-        'log_contact_status_changes': '',
+            'requests ("wizz").'),
         'muc_highlight_words': _(
             'A list of words (semicolon separated) that will be '
             'highlighted in group chats.'),
         'muclumbus_api_http_uri': '',
         'muclumbus_api_jid': '',
         'muclumbus_api_pref': _(
-            'API Preferences. Possible values: \'http\', \'iq\''),
+            'API Preferences. Possible values: "http", "iq"'),
         'notification_preview_message': _(
             'Preview new messages in notification popup?'),
         'notification_timeout': '',
         'notify_on_all_muc_messages': '',
         'plugins_repository_enabled': _(
             'If enabled, Gajim offers to download plugins hosted on gajim.org'),
-        'restore_timeout': _(
-            'How far back in time (minutes) chat history is restored. -1 '
-            'means no limit.'),
         'save_main_window_position': _(
             'If enabled, Gajim will save the main window position when hiding '
             'it, and restore it when showing the window again.'),
         'search_engine': '',
         'send_on_ctrl_enter': _(
             'Send message on Ctrl+Enter and make a new line with Enter.'),
-        'shell_like_completion': _(
-            'If enabled, completion in group chats will be like a shell '
-            'auto-completion.'),
+        'show_chatstate_in_banner': _(
+            'Show chat state (e.g. "is typing…") next to your contact’s name'),
         'stun_server': _('STUN server to use when using Jingle'),
         'trayicon_notification_on_events': _(
             'Notify of events in the notification area.'),
@@ -710,7 +696,7 @@ ADVANCED_SETTINGS = {
             'Enables libappindicator, an alternative tray icon provider'),
         'use_stun_server': _(
             'If enabled, Gajim will try to use a STUN server when using Jingle.'
-            ' The one in \'stun_server\' option, or the one given by '
+            ' The one in "stun_server" option, or the one given by '
             'the XMPP server.'),
         'use_urgency_hint': _(
             'If enabled, Gajim makes the window flash (the default behaviour '
