@@ -41,10 +41,13 @@ from gajim import IS_PORTABLE
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import optparser
-from gajim.common.helpers import get_muc_context
 from gajim.common.storage.base import Encoder
 from gajim.common.storage.base import json_decoder
 from gajim.common.setting_values import APP_SETTINGS
+from gajim.common.setting_values import AllContactSettings
+from gajim.common.setting_values import AllContactSettingsT
+from gajim.common.setting_values import BoolContactSettings
+from gajim.common.setting_values import StringContactSettings
 from gajim.common.setting_values import AllGroupChatSettings
 from gajim.common.setting_values import AllGroupChatSettingsT
 from gajim.common.setting_values import BoolGroupChatSettings
@@ -52,7 +55,7 @@ from gajim.common.setting_values import IntGroupChatSettings
 from gajim.common.setting_values import StringGroupChatSettings
 from gajim.common.setting_values import AllWorkspaceSettings
 from gajim.common.setting_values import AllWorkspaceSettingsT
-from gajim.common.setting_values import OpenChatSettingT
+from gajim.common.setting_values import OpenChatsSettingT
 from gajim.common.setting_values import StringWorkspaceSettings
 from gajim.common.setting_values import ACCOUNT_SETTINGS
 from gajim.common.setting_values import PROXY_SETTINGS
@@ -78,6 +81,8 @@ SETTING_TYPE = Union[bool, int, str, object]
 
 log = logging.getLogger('gajim.c.settings')
 
+CURRENT_USER_VERSION = 3
+
 CREATE_SQL = '''
     CREATE TABLE settings (
             name TEXT UNIQUE,
@@ -96,10 +101,11 @@ CREATE_SQL = '''
     INSERT INTO settings(name, settings) VALUES ('plugins', '{}');
     INSERT INTO settings(name, settings) VALUES ('workspaces', '%s');
 
-    PRAGMA user_version=1;
+    PRAGMA user_version=%s;
     ''' % (json.dumps(STATUS_PRESET_EXAMPLES),
            json.dumps(PROXY_EXAMPLES),
-           json.dumps(INITAL_WORKSPACE))
+           json.dumps(INITAL_WORKSPACE),
+           CURRENT_USER_VERSION)
 
 
 _SignalCallable = Callable[[Any, str, Optional[str], Optional[JID]], Any]
@@ -146,19 +152,20 @@ class Settings:
                     handlers.remove(handler)
 
     def bind_signal(self,
-                    setting,
-                    widget,
-                    func_name,
-                    account=None,
-                    jid=None,
-                    inverted=False,
-                    default_text=None):
+                    setting: str,
+                    widget: Any,
+                    func_name: str,
+                    account: Optional[str] = None,
+                    jid: Optional[JID] = None,
+                    inverted: bool = False,
+                    default_text: Optional[str] = None
+                    ) -> None:
 
         callbacks = self._callbacks[(setting, account, jid)]
         func = getattr(widget, func_name)
         callbacks.append((func, inverted, default_text))
 
-        def _on_destroy(*args):
+        def _on_destroy(*args: Any) -> None:
             callbacks.remove((func, inverted, default_text))
 
         widget.connect('destroy', _on_destroy)
@@ -212,6 +219,7 @@ class Settings:
             self._commit()
         self._migrate_database()
         self._load_app_overrides()
+        self._commit()
 
     @staticmethod
     def _setup_installation_defaults() -> None:
@@ -315,6 +323,31 @@ class Settings:
             self._settings['workspaces'] = INITAL_WORKSPACE
             self._set_user_version(1)
 
+        if version < 2:
+            # Migrate open chats to new key and format
+            for workspace in self._settings['workspaces'].values():
+                open_chats = []
+                for open_chat in workspace.get('open_chats', []):
+                    account, jid, type_, pinned = open_chat
+                    open_chats.append({'account': account,
+                                       'jid': jid,
+                                       'type': type_,
+                                       'pinned': pinned,
+                                       'position': -1})
+
+                workspace['chats'] = open_chats
+                workspace.pop('open_chats', None)
+
+            self._set_user_version(2)
+
+        if version < 3:
+            # Migrate open chats to new key and format
+            for account_settings in self._account_settings.values():
+                if account_settings['account'].get('active') is None:
+                    account_settings['account']['active'] = True
+
+            self._set_user_version(3)
+
     def _migrate_old_config(self) -> None:
         config_file = configpaths.get('CONFIG_FILE')
         if not config_file.exists():
@@ -580,6 +613,12 @@ class Settings:
                         value: Optional[int]) -> None:
         ...
 
+    @overload
+    def set_app_setting(self,
+                        setting: Literal['workspace_order'],
+                        value: list[str]) -> None:
+        ...
+
     def set_app_setting(self,
                         setting: str,
                         value: Optional[AllSettingsT]) -> None:
@@ -810,7 +849,9 @@ class Settings:
             return self._account_settings[account]['group_chat'][jid][setting]
         except KeyError:
 
-            context = get_muc_context(jid)
+            client = app.get_client(account)
+            contact = client.get_module('Contacts').get_contact(jid)
+            context = contact.muc_context
             if context is None:
                 # If there is no disco info available
                 # to determine the context assume public
@@ -872,13 +913,16 @@ class Settings:
 
         default = ACCOUNT_SETTINGS['group_chat'][setting]
         if default in (HAS_APP_DEFAULT, HAS_ACCOUNT_DEFAULT):
-
-            context = get_muc_context(jid)
-            if context is None:
-                # If there is no disco info available
-                # to determine the context assume public
-                log.warning('Unable to determine context for: %s', jid)
-                context = 'public'
+            context = 'public'
+            if app.account_is_connected(account):
+                client = app.get_client(account)
+                contact = client.get_module('Contacts').get_contact(jid)
+                context = contact.muc_context
+                if context is None:
+                    # If there is no disco info available
+                    # to determine the context assume public
+                    log.warning('Unable to determine context for: %s', jid)
+                    context = 'public'
 
             default_store = APP_SETTINGS
             if default is HAS_ACCOUNT_DEFAULT:
@@ -914,19 +958,39 @@ class Settings:
     def set_group_chat_settings(self,
                                 setting: str,
                                 value: SETTING_TYPE,
-                                context: str = None) -> None:
+                                context: Optional[str] = None
+                                ) -> None:
 
         for account, acc_settings in self._account_settings.items():
             for jid in acc_settings['group_chat']:
                 if context is not None:
-                    if get_muc_context(jid) != context:
+                    client = app.get_client(account)
+                    contact = client.get_module('Contacts').get_contact(jid)
+                    if contact.muc_context != context:
                         continue
                 self.set_group_chat_setting(account, jid, setting, value)
 
+    @overload
+    def get_contact_setting(self,
+                            account: str,
+                            jid: JID,
+                            setting: BoolContactSettings
+                            ) -> bool:
+        ...
+
+    @overload
+    def get_contact_setting(self,
+                            account: str,
+                            jid: JID,
+                            setting: StringContactSettings
+                            ) -> str:
+        ...
+
     def get_contact_setting(self,
                             account: str,
                             jid: JID,
-                            setting: str) -> SETTING_TYPE:
+                            setting: AllContactSettings
+                            ) -> AllContactSettingsT:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -946,11 +1010,27 @@ class Settings:
 
             return default
 
+    @overload
     def set_contact_setting(self,
                             account: str,
                             jid: JID,
-                            setting: str,
-                            value: SETTING_TYPE) -> None:
+                            setting: StringContactSettings,
+                            value: str) -> None:
+        ...
+
+    @overload
+    def set_contact_setting(self,
+                            account: str,
+                            jid: JID,
+                            setting: BoolContactSettings,
+                            value: bool) -> None:
+        ...
+
+    def set_contact_setting(self,
+                            account: str,
+                            jid: JID,
+                            setting: AllContactSettings,
+                            value: AllContactSettingsT) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -1136,8 +1216,8 @@ class Settings:
     @overload
     def set_workspace_setting(self,
                               workspace_id: str,
-                              setting: Literal['open_chats'],
-                              value: list[tuple[str, JID, str, bool]]
+                              setting: Literal['chats'],
+                              value: OpenChatsSettingT
                               ) -> None:
         ...
 
@@ -1163,8 +1243,8 @@ class Settings:
     @overload
     def get_workspace_setting(self,
                               workspace_id: str,
-                              setting: Literal['open_chats']
-                              ) -> OpenChatSettingT:
+                              setting: Literal['chats']
+                              ) -> OpenChatsSettingT:
         ...
 
     @overload
@@ -1189,6 +1269,9 @@ class Settings:
         except KeyError:
             return WORKSPACE_SETTINGS[setting]
 
+    def get_workspace_count(self) -> int:
+        return len(self._settings['workspaces'].keys())
+
     def get_workspaces(self) -> List[str]:
         workspace_order = app.settings.get_app_setting('workspace_order')
 
