@@ -26,6 +26,8 @@ from pathlib import Path
 
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
+from gi.repository import Pango
+from gi.repository import PangoCairo
 import cairo
 from nbxmpp.protocol import JID
 
@@ -38,11 +40,10 @@ from gajim.common.const import AvatarSize
 from gajim.common.const import StyleAttr
 
 from .const import DEFAULT_WORKSPACE_COLOR
-from .emoji_data import is_emoji
-from .emoji_data import get_emoji_font
+from .util import get_contact_color
+from .util import get_first_graphemes
 from .util import load_icon_surface
 from .util import load_pixbuf
-from .util import text_to_color
 from .util import scale_with_ratio
 from .util import get_css_show_class
 from .util import convert_rgb_string_to_float
@@ -52,8 +53,12 @@ from .util import make_rgba
 log = logging.getLogger('gajim.gui.avatar')
 
 
-AvatarCacheT = dict[JID, dict[tuple[int, int, Optional[str]],
-                              cairo.ImageSurface]]
+AvatarCacheT = dict[Union[JID, str], dict[tuple[int, int, Optional[str]],
+                                          cairo.ImageSurface]]
+
+
+def generate_avatar_letter(text: str) -> str:
+    return get_first_graphemes(text.lstrip(), 1).upper()
 
 
 def generate_avatar(letters: str,
@@ -70,7 +75,7 @@ def generate_avatar(letters: str,
 
     width = size
     height = size
-    font_size = size * 0.5
+    font_size = size * 0.4
 
     # Set up surface
     surface = cairo.ImageSurface(cairo.Format.ARGB32, width, height)
@@ -81,38 +86,36 @@ def generate_avatar(letters: str,
     context.fill()
 
     # Draw letters
-    if is_emoji(letters):
-        font_face = get_emoji_font()
-    else:
-        font_face = 'sans-serif'
-    context.select_font_face(font_face,
-                             cairo.FontSlant.NORMAL,
-                             cairo.FontWeight.NORMAL)
-    context.set_font_size(font_size)
-    extends = context.text_extents(letters)
-    x_bearing = extends.x_bearing
-    y_bearing = extends.y_bearing
-    ex_width = extends.width
-    ex_height = extends.height
+    layout = PangoCairo.create_layout(context)
+    layout.set_text(generate_avatar_letter(letters))
+
+    description = Pango.FontDescription()
+    description.set_family('Sans')
+    description.set_size(int(font_size * Pango.SCALE))
+
+    layout.set_font_description(description)
+
+    _ink_rect, logical_rect = layout.get_extents()
+    layout_width = logical_rect.width / Pango.SCALE
+    layout_height = logical_rect.height / Pango.SCALE
+    x_pos = (width - layout_width) / 2
+    y_pos = (height - layout_height) / 2
 
-    x_pos = width / 2 - (ex_width / 2 + x_bearing)
-    y_pos = height / 2 - (ex_height / 2 + y_bearing)
     context.move_to(x_pos, y_pos)
     context.set_source_rgb(0.95, 0.95, 0.95)
     context.set_operator(cairo.Operator.OVER)
-    context.show_text(letters)
+    PangoCairo.show_layout(context, layout)
 
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 @lru_cache(maxsize=None)
 def generate_default_avatar(letter: str,
-                            color_string: str,
+                            color: tuple[float, float, float],
                             size: int,
                             scale: int,
                             style: str = 'circle') -> cairo.ImageSurface:
 
-    color = text_to_color(color_string)
     surface = generate_avatar(letter, color, size, scale)
     surface = clip(surface, style)
     surface.set_device_scale(scale, scale)
@@ -132,7 +135,8 @@ def make_workspace_avatar(letter: str,
 
 
 def add_status_to_avatar(surface: cairo.ImageSurface,
-                         show: str) -> cairo.ImageSurface:
+                         show: str
+                         ) -> cairo.ImageSurface:
 
     width = surface.get_width()
     height = surface.get_height()
@@ -179,11 +183,15 @@ def add_status_to_avatar(surface: cairo.ImageSurface,
         context.set_line_width(clip_radius / 4)
         context.stroke()
 
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 @lru_cache(maxsize=128)
-def get_show_circle(show, size: int, scale: int) -> cairo.ImageSurface:
+def get_show_circle(show: Union[str, types.PresenceShowT],
+                    size: int,
+                    scale: int
+                    ) -> cairo.ImageSurface:
+
     if not isinstance(show, str):
         show = show.value
 
@@ -212,7 +220,7 @@ def get_show_circle(show, size: int, scale: int) -> cairo.ImageSurface:
         context.set_line_width(size / 10)
         context.stroke()
 
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 def fit(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
@@ -235,7 +243,7 @@ def fit(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
     context.scale(factor, factor)
     context.set_source_surface(square_surface, 0, 0)
     context.paint()
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 def square(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
@@ -259,7 +267,7 @@ def square(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
 
     context.set_source_surface(surface, x_pos, y_pos)
     context.paint()
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 def clip(surface: cairo.ImageSurface, mode: str) -> cairo.ImageSurface:
@@ -289,7 +297,7 @@ def clip_circle(surface: cairo.ImageSurface) -> cairo.ImageSurface:
     context.clip()
     context.paint()
 
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 def round_corners(surface: cairo.ImageSurface) -> cairo.ImageSurface:
@@ -320,7 +328,7 @@ def round_corners(surface: cairo.ImageSurface) -> cairo.ImageSurface:
 
     context.paint()
 
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 def convert_to_greyscale(surface: cairo.ImageSurface) -> cairo.ImageSurface:
@@ -333,14 +341,14 @@ def convert_to_greyscale(surface: cairo.ImageSurface) -> cairo.ImageSurface:
     context.set_source_rgba(1, 1, 1, 0.5)
     context.rectangle(0, 0, surface.get_width(), surface.get_height())
     context.fill()
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 class AvatarStorage(metaclass=Singleton):
     def __init__(self):
         self._cache: AvatarCacheT = defaultdict(dict)
 
-    def invalidate_cache(self, jid: JID) -> None:
+    def invalidate_cache(self, jid: Union[JID, str]) -> None:
         self._cache.pop(jid, None)
 
     def get_pixbuf(self,
@@ -356,7 +364,11 @@ class AvatarStorage(metaclass=Singleton):
 
         surface = self.get_surface(
             contact, size, scale, show, default, transport_icon, style)
-        return Gdk.pixbuf_get_from_surface(surface, 0, 0, size, size)
+        return Gdk.pixbuf_get_from_surface(surface,
+                                           0,
+                                           0,
+                                           size * scale,
+                                           size * scale)
 
     def get_surface(self,
                     contact: Union[types.BareContact,
@@ -373,6 +385,7 @@ class AvatarStorage(metaclass=Singleton):
 
         if transport_icon is not None:
             surface = load_icon_surface(transport_icon, size, scale)
+            assert surface is not None
             if show is not None:
                 surface = add_status_to_avatar(surface, show)
             self._cache[jid][(size, scale, show)] = surface
@@ -391,11 +404,10 @@ class AvatarStorage(metaclass=Singleton):
                 return surface
 
         name = contact.name
-        color_string = str(contact.jid)
-
-        letter = self._generate_letter(name)
+        color = get_contact_color(contact)
+        letter = generate_avatar_letter(name)
         surface = generate_default_avatar(
-            letter, color_string, size, scale, style=style)
+            letter, color, size, scale, style=style)
         if show is not None:
             surface = add_status_to_avatar(surface, show)
         self._cache[jid][(size, scale, show)] = surface
@@ -412,11 +424,12 @@ class AvatarStorage(metaclass=Singleton):
 
         if transport_icon is not None:
             surface = load_icon_surface(transport_icon, size, scale)
-            self._cache[jid][(size, scale)] = surface
+            assert surface is not None
+            self._cache[jid][(size, scale, None)] = surface
             return surface
 
         if not default:
-            surface = self._cache[jid].get((size, scale))
+            surface = self._cache[jid].get((size, scale, None))
             if surface is not None:
                 return surface
 
@@ -425,18 +438,21 @@ class AvatarStorage(metaclass=Singleton):
                 surface = self.surface_from_filename(avatar_sha, size, scale)
                 if surface is not None:
                     surface = clip(surface, style)
-                    self._cache[jid][(size, scale)] = surface
+                    self._cache[jid][(size, scale, None)] = surface
                     return surface
 
                 # avatar_sha set, but image is missing
                 # (e.g. avatar cache deleted)
                 app.storage.cache.set_muc(jid, 'avatar', None)
 
-        con = app.connections[account]
-        name = get_groupchat_name(con, jid)
-        letter = self._generate_letter(name)
-        surface = generate_default_avatar(letter, str(jid), size, scale, style)
-        self._cache[jid][(size, scale)] = surface
+        client = app.get_client(account)
+        contact = client.get_module('Contacts').get_bare_contact(jid)
+
+        name = get_groupchat_name(client, jid)
+        color = get_contact_color(contact)
+        letter = generate_avatar_letter(name)
+        surface = generate_default_avatar(letter, color, size, scale, style)
+        self._cache[jid][(size, scale, None)] = surface
         return surface
 
     def get_workspace_surface(self,
@@ -444,7 +460,7 @@ class AvatarStorage(metaclass=Singleton):
                               size: int,
                               scale: int) -> Optional[cairo.ImageSurface]:
 
-        surface = self._cache[workspace_id].get((size, scale))
+        surface = self._cache[workspace_id].get((size, scale, None))
         if surface is not None:
             return surface
 
@@ -462,10 +478,9 @@ class AvatarStorage(metaclass=Singleton):
             app.settings.set_workspace_setting(workspace_id, 'avatar_sha', '')
 
         rgba = make_rgba(color or DEFAULT_WORKSPACE_COLOR)
-        letter = name[:1].upper()
         surface = make_workspace_avatar(
-            letter, rgba_to_float(rgba), size, scale)
-        self._cache[workspace_id][(size, scale)] = surface
+            name, rgba_to_float(rgba), size, scale)
+        self._cache[workspace_id][(size, scale, None)] = surface
         return surface
 
     @staticmethod
@@ -483,17 +498,18 @@ class AvatarStorage(metaclass=Singleton):
                                          height,
                                          GdkPixbuf.InterpType.BILINEAR)
 
+        assert pixbuf is not None
         return pixbuf.save_to_bufferv('png', [], [])
 
     @staticmethod
     def save_avatar(data: bytes) -> Optional[str]:
-        """
+        '''
         Save an avatar to the harddisk
 
         :param data:  bytes
 
         returns SHA1 value of the avatar or None on error
-        """
+        '''
         if data is None:
             return None
 
@@ -563,10 +579,3 @@ class AvatarStorage(metaclass=Singleton):
         if surface is None:
             return None
         return clip(surface, style)
-
-    @staticmethod
-    def _generate_letter(name: str) -> str:
-        for letter in name:
-            if letter.isalpha():
-                return letter.capitalize()
-        return name[0].capitalize()
