@@ -15,6 +15,7 @@
 from __future__ import annotations
 
 from typing import Optional
+from typing import Union
 
 from datetime import datetime
 from datetime import timedelta
@@ -24,13 +25,13 @@ from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import GLib
 from gi.repository import Gtk
-from gi.repository import GObject
 from gi.repository import Pango
 
 import cairo
 
 from nbxmpp.errors import StanzaError
 from nbxmpp.modules.security_labels import Displaymarking
+from nbxmpp.structs import CommonError
 
 from gajim.common import app
 from gajim.common.const import AvatarSize
@@ -39,41 +40,30 @@ from gajim.common.const import TRUST_SYMBOL_DATA
 from gajim.common.helpers import AdditionalDataDict
 from gajim.common.helpers import from_one_line
 from gajim.common.helpers import get_group_chat_nick
-from gajim.common.helpers import get_muc_context
 from gajim.common.helpers import message_needs_highlight
 from gajim.common.helpers import to_user_string
 from gajim.common.i18n import _
-from gajim.common.i18n import Q_
+from gajim.common.i18n import is_rtl_text
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.types import ChatContactT
 
 from .base import BaseRow
+from .widgets import AvatarBox
+from .widgets import DateTimeLabel
+from .widgets import NicknameLabel
+from .widgets import MessageIcons
 from .widgets import MoreMenuButton
 from ..message_widget import MessageWidget
 from ...dialogs import InputDialog
 from ...dialogs import DialogButton
 from ...preview import PreviewWidget
 from ...util import format_fingerprint
-from ...util import get_cursor
+
 
 MERGE_TIMEFRAME = timedelta(seconds=120)
 
 
 class MessageRow(BaseRow):
-
-    __gsignals__ = {
-        'mention': (
-            GObject.SignalFlags.RUN_LAST,
-            None,
-            (str,)
-        ),
-        'quote': (
-            GObject.SignalFlags.RUN_LAST,
-            None,
-            (str,)
-        ),
-    }
-
     def __init__(self,
                  account: str,
                  contact: ChatContactT,
@@ -86,8 +76,7 @@ class MessageRow(BaseRow):
                  additional_data: Optional[AdditionalDataDict] = None,
                  display_marking: Optional[Displaymarking] = None,
                  marker: Optional[str] = None,
-                 error: Optional[StanzaError] = None,
-                 encryption_enabled: bool = False,
+                 error: Union[CommonError, StanzaError, None] = None,
                  log_line_id: Optional[int] = None) -> None:
 
         BaseRow.__init__(self, account)
@@ -103,6 +92,8 @@ class MessageRow(BaseRow):
         self.additional_data = additional_data
         self.display_marking = display_marking
 
+        self.set_selectable(True)
+
         self._account = account
         self._contact = contact
 
@@ -118,21 +109,23 @@ class MessageRow(BaseRow):
 
         if self._is_groupchat:
             our_nick = get_group_chat_nick(self._account, self._contact.jid)
-            is_self = name == our_nick
+            from_us = name == our_nick
         else:
-            is_self = kind == 'outgoing'
+            from_us = kind == 'outgoing'
 
         is_previewable = False
-        if additional_data is not None:
+        preview_enabled = app.settings.get('enable_file_preview')
+        if additional_data is not None and preview_enabled:
             is_previewable = app.preview_manager.is_previewable(
                 text, additional_data)
+
         if is_previewable:
-            context = None
-            if self._is_groupchat:
-                context = get_muc_context(self._contact.jid)
+            muc_context = None
+            if isinstance(self._contact, GroupchatContact):
+                muc_context = self._contact.muc_context
             self._message_widget = PreviewWidget(account)
             app.preview_manager.create_preview(
-                text, self._message_widget, is_self, context)
+                text, self._message_widget, from_us, muc_context)
         else:
             self._message_widget = MessageWidget(account)
             self._message_widget.add_with_styling(text, nickname=name)
@@ -145,36 +138,20 @@ class MessageRow(BaseRow):
         if self._contact.jid == self._client.get_own_jid().bare:
             name = _('Me')
 
-        name_widget = self.create_name_widget(name, is_self)
+        name_widget = NicknameLabel(name, from_us)
 
         self._meta_box = Gtk.Box(spacing=6)
         self._meta_box.set_hexpand(True)
         self._meta_box.pack_start(name_widget, False, True, 0)
-        timestamp_label = self.create_timestamp_widget(self.timestamp)
+        timestamp_label = DateTimeLabel(self.timestamp)
         self._meta_box.pack_start(timestamp_label, False, True, 0)
 
-        if kind in ('incoming', 'incoming_queue', 'outgoing'):
-            if additional_data is not None:
-                encryption_img = self._get_encryption_image(
-                    additional_data, encryption_enabled)
-                if encryption_img:
-                    self._meta_box.pack_start(encryption_img, False, True, 0)
+        if additional_data is not None:
+            encryption_img = self._get_encryption_image(additional_data)
+            if encryption_img:
+                self._meta_box.pack_start(encryption_img, False, True, 0)
 
-        if display_marking and app.settings.get_account_setting(
-                account, 'enable_security_labels'):
-            label_text = GLib.markup_escape_text(display_marking.name)
-            if label_text:
-                display_marking_label = Gtk.Label()
-                display_marking_label.set_ellipsize(Pango.EllipsizeMode.END)
-                display_marking_label.set_max_width_chars(30)
-                display_marking_label.set_tooltip_text(label_text)
-                bgcolor = display_marking.bgcolor
-                fgcolor = display_marking.fgcolor
-                label_text = (
-                    f'<span size="small" bgcolor="{bgcolor}" '
-                    f'fgcolor="{fgcolor}"><tt>{label_text}</tt></span>')
-                display_marking_label.set_markup(label_text)
-                self._meta_box.add(display_marking_label)
+        self._add_security_label(display_marking)
 
         self._message_icons = MessageIcons()
 
@@ -203,34 +180,59 @@ class MessageRow(BaseRow):
                 self.set_receipt()
 
         self._meta_box.pack_start(self._message_icons, False, True, 0)
-        avatar = self._get_avatar(kind, name)
-        self._avatar_image = Gtk.Image.new_from_surface(avatar)
 
-        if self._is_groupchat:
-            avatar_placeholder = Gtk.EventBox()
-            avatar_placeholder.connect(
-                'button-press-event', self._on_avatar_clicked, name)
-            avatar_placeholder.connect('realize', self._on_realize)
-        else:
-            avatar_placeholder = Gtk.Box()
-        avatar_placeholder.set_size_request(AvatarSize.ROSTER, -1)
-        avatar_placeholder.set_valign(Gtk.Align.START)
-        avatar_placeholder.add(self._avatar_image)
+        avatar = self._get_avatar(kind, name)
+        self._avatar_box = AvatarBox(self._contact, name, avatar)
 
         self._bottom_box = Gtk.Box(spacing=6)
         self._bottom_box.add(self._message_widget)
 
+        if is_rtl_text(text):
+            self._bottom_box.set_halign(Gtk.Align.END)
+            self._message_widget.set_direction(Gtk.TextDirection.RTL)
+
         more_menu_button = MoreMenuButton(self, self._contact, name)
-        more_menu_button.set_hexpand(True)
-        more_menu_button.set_halign(Gtk.Align.END)
         self._bottom_box.pack_end(more_menu_button, False, True, 0)
 
-        self.grid.attach(avatar_placeholder, 0, 0, 1, 2)
+        self.grid.attach(self._avatar_box, 0, 0, 1, 2)
         self.grid.attach(self._meta_box, 1, 0, 1, 1)
         self.grid.attach(self._bottom_box, 1, 1, 1, 1)
 
         self.show_all()
 
+    def enable_selection_mode(self) -> None:
+        if isinstance(self._message_widget, MessageWidget):
+            self._message_widget.set_selectable(False)
+
+    def disable_selection_mode(self) -> None:
+        if isinstance(self._message_widget, MessageWidget):
+            self._message_widget.set_selectable(True)
+
+    def _add_security_label(self,
+                            display_marking: Optional[Displaymarking]
+                            ) -> None:
+
+        if display_marking is None:
+            return
+
+        if not app.settings.get_account_setting(self._account,
+                                                'enable_security_labels'):
+            return
+
+        label_text = GLib.markup_escape_text(display_marking.name)
+        if label_text:
+            display_marking_label = Gtk.Label()
+            display_marking_label.set_ellipsize(Pango.EllipsizeMode.END)
+            display_marking_label.set_max_width_chars(30)
+            display_marking_label.set_tooltip_text(label_text)
+            bgcolor = display_marking.bgcolor
+            fgcolor = display_marking.fgcolor
+            label_text = (
+                f'<span size="small" bgcolor="{bgcolor}" '
+                f'fgcolor="{fgcolor}"><tt>{label_text}</tt></span>')
+            display_marking_label.set_markup(label_text)
+            self._meta_box.add(display_marking_label)
+
     def _check_for_highlight(self, text: str) -> None:
         assert isinstance(self._contact, GroupchatContact)
         if self._contact.nickname is None:
@@ -263,21 +265,6 @@ class MessageRow(BaseRow):
         assert not isinstance(avatar, GdkPixbuf.Pixbuf)
         return avatar
 
-    def _on_avatar_clicked(self,
-                           _widget: Gtk.Widget,
-                           event: Gdk.EventButton,
-                           name: str
-                           ) -> int:
-        if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 1:
-            self.emit('mention', name)
-        return Gdk.EVENT_STOP
-
-    @staticmethod
-    def _on_realize(event_box: Gtk.EventBox) -> None:
-        window = event_box.get_window()
-        if window is not None:
-            window.set_cursor(get_cursor('pointer'))
-
     def is_same_sender(self, message: MessageRow) -> bool:
         return message.name == self.name
 
@@ -322,16 +309,23 @@ class MessageRow(BaseRow):
             return False
         return abs(message.timestamp - self.timestamp) < MERGE_TIMEFRAME
 
+    def get_text(self) -> str:
+        return self._message_widget.get_text()
+
     def on_copy_message(self, _widget: Gtk.Widget) -> None:
         time_format = from_one_line(app.settings.get('chat_timestamp_format'))
         timestamp_formatted = self.timestamp.strftime(time_format)
 
         clip = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-        text = self._message_widget.get_text()
+        text = self.get_text()
         clip.set_text(f'{timestamp_formatted} - {self.name}: {text}', -1)
 
     def on_quote_message(self, _widget: Gtk.Widget) -> None:
-        self.emit('quote', self._message_widget.get_text())
+        app.window.activate_action(
+            'quote', GLib.Variant('s', self.get_text()))
+
+    def on_correct_message(self, _widget: Gtk.Widget) -> None:
+        app.window.activate_action('correct-message', None)
 
     def on_retract_message(self, _widget: Gtk.Widget) -> None:
         def _on_retract(reason: str) -> None:
@@ -351,12 +345,12 @@ class MessageRow(BaseRow):
 
     def _get_encryption_image(self,
                               additional_data: AdditionalDataDict,
-                              encryption_enabled: bool = False
                               ) -> Optional[Gtk.Image]:
+
         details = self._get_encryption_details(additional_data)
         if details is None:
             # Message was not encrypted
-            if not encryption_enabled:
+            if not self._contact.settings.get('encryption'):
                 return None
             icon = 'channel-insecure-symbolic'
             color = 'unencrypted-color'
@@ -416,6 +410,13 @@ class MessageRow(BaseRow):
             self._message_widget.destroy()
             self._message_widget = MessageWidget(self._account)
             self._bottom_box.pack_start(self._message_widget, True, True, 0)
+            if is_rtl_text(text):
+                self._bottom_box.set_halign(Gtk.Align.END)
+                self._message_widget.set_direction(Gtk.TextDirection.RTL)
+            else:
+                self._bottom_box.set_halign(Gtk.Align.FILL)
+                self._message_widget.set_direction(Gtk.TextDirection.LTR)
+
         self._message_widget.add_with_styling(text)
         self.get_style_context().add_class('retracted-message')
 
@@ -423,6 +424,13 @@ class MessageRow(BaseRow):
         if not isinstance(self._message_widget, PreviewWidget):
             self._message_widget.add_with_styling(text, nickname)
 
+            if is_rtl_text(text):
+                self._bottom_box.set_halign(Gtk.Align.END)
+                self._message_widget.set_direction(Gtk.TextDirection.RTL)
+            else:
+                self._bottom_box.set_halign(Gtk.Align.FILL)
+                self._message_widget.set_direction(Gtk.TextDirection.LTR)
+
         self._has_receipt = False
         self._message_icons.set_receipt_icon_visible(False)
         self._message_icons.set_correction_icon_visible(True)
@@ -440,63 +448,17 @@ class MessageRow(BaseRow):
 
     def update_avatar(self) -> None:
         avatar = self._get_avatar(self.kind, self.name)
-        self._avatar_image.set_from_surface(avatar)
+        self._avatar_box.set_from_surface(avatar)
 
     def set_merged(self, merged: bool) -> None:
         self._merged = merged
         if merged:
             self.get_style_context().add_class('merged')
-            self._avatar_image.set_no_show_all(True)
-            self._avatar_image.hide()
             self._meta_box.set_no_show_all(True)
             self._meta_box.hide()
         else:
             self.get_style_context().remove_class('merged')
-            self._avatar_image.set_no_show_all(False)
-            self._avatar_image.show()
             self._meta_box.set_no_show_all(False)
             self._meta_box.show()
 
-
-class MessageIcons(Gtk.Box):
-    def __init__(self) -> None:
-        Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
-
-        self._correction_image = Gtk.Image.new_from_icon_name(
-            'document-edit-symbolic', Gtk.IconSize.MENU)
-        self._correction_image.set_no_show_all(True)
-        self._correction_image.get_style_context().add_class('dim-label')
-
-        self._marker_image = Gtk.Image()
-        self._marker_image.set_no_show_all(True)
-        self._marker_image.get_style_context().add_class('dim-label')
-
-        self._error_image = Gtk.Image.new_from_icon_name(
-            'dialog-warning-symbolic', Gtk.IconSize.MENU)
-        self._error_image.get_style_context().add_class('warning-color')
-        self._error_image.set_no_show_all(True)
-
-        self.add(self._correction_image)
-        self.add(self._marker_image)
-        self.add(self._error_image)
-        self.show_all()
-
-    def set_receipt_icon_visible(self, visible: bool) -> None:
-        if not app.settings.get('positive_184_ack'):
-            return
-        self._marker_image.set_visible(visible)
-        self._marker_image.set_from_icon_name(
-            'feather-check-symbolic', Gtk.IconSize.MENU)
-        self._marker_image.set_tooltip_text(Q_('?Message state:Received'))
-
-    def set_correction_icon_visible(self, visible: bool) -> None:
-        self._correction_image.set_visible(visible)
-
-    def set_correction_tooltip(self, text: str) -> None:
-        self._correction_image.set_tooltip_markup(text)
-
-    def set_error_icon_visible(self, visible: bool) -> None:
-        self._error_image.set_visible(visible)
-
-    def set_error_tooltip(self, text: str) -> None:
-        self._error_image.set_tooltip_markup(text)
+        self._avatar_box.set_merged(merged)
