@@ -28,6 +28,7 @@ from functools import partial
 from enum import IntEnum
 from enum import unique
 from datetime import datetime
+from datetime import timezone
 
 from gi.repository import Gtk
 from gi.repository import Gdk
@@ -173,16 +174,24 @@ class FileTransfersWindow:
 
         self._ui.connect_signals(self)
 
-        # pylint: disable=line-too-long
-        app.ged.register_event_handler('file-completed', ged.GUI1, self._file_completed)
-        app.ged.register_event_handler('file-hash-error', ged.GUI1, self._file_hash_error)
-        app.ged.register_event_handler('file-send-error', ged.GUI1, self._file_send_error)
-        app.ged.register_event_handler('file-request-error', ged.GUI1, self._file_request_error)
-        app.ged.register_event_handler('file-request-received', ged.GUI1, self._file_request_received)
-        app.ged.register_event_handler('file-progress', ged.GUI1, self._file_progress)
-        app.ged.register_event_handler('file-error', ged.GUI1, self._file_error)
-        app.ged.register_event_handler('jingle-ft-cancelled-received', ged.GUI1, self._file_transfer_cancelled)
-        # pylint: enable=line-too-long
+        app.ged.register_event_handler(
+            'file-completed', ged.GUI1, self._file_completed)
+        app.ged.register_event_handler(
+            'file-hash-error', ged.GUI1, self._file_hash_error)
+        app.ged.register_event_handler(
+            'file-send-error', ged.GUI1, self._file_send_error)
+        app.ged.register_event_handler(
+            'file-request-error', ged.GUI1, self._file_request_error)
+        app.ged.register_event_handler(
+            'file-request-received', ged.GUI1, self._file_request_received)
+        app.ged.register_event_handler(
+            'file-progress', ged.GUI1, self._file_progress)
+        app.ged.register_event_handler(
+            'file-error', ged.GUI1, self._file_error)
+        app.ged.register_event_handler(
+            'jingle-ft-cancelled-received',
+            ged.GUI1,
+            self._file_transfer_cancelled)
 
     def _file_completed(self, event):
         self.set_status(event.file_props, 'ok')
@@ -319,9 +328,9 @@ class FileTransfersWindow:
         return value
 
     def find_transfer_by_jid(self, account, jid):
-        """
+        '''
         Find all transfers with peer 'jid' that belong to 'account'
-        """
+        '''
         active_transfers = [[], []]  # ['senders', 'receivers']
         allfp = FilesProp.getAllFileProp()
         for file_props in allfp:
@@ -409,9 +418,9 @@ class FileTransfersWindow:
                   file_path: str,
                   file_desc: str = ''
                   ) -> bool:
-        """
+        '''
         Start the real transfer(upload) of the file
-        """
+        '''
         if file_is_locked(file_path):
             pritext = _('Gajim can not read this file')
             sextext = _('Another process is using this file.')
@@ -474,7 +483,7 @@ class FileTransfersWindow:
                     file_name = GLib.markup_escape_text(
                         os.path.basename(file_path))
                     ErrorDialog(
-                        _('Cannot overwrite existing file \'%s\'') % file_name,
+                        _('Cannot overwrite existing file "%s"') % file_name,
                         _('A file with this name already exists and you do '
                           'not have permission to overwrite it.'))
                     return
@@ -486,7 +495,7 @@ class FileTransfersWindow:
                 # windows, not to mark that a folder is read-only.
                 # See ticket #3587
                 ErrorDialog(
-                    _('Directory \'%s\' is not writable') % dirname,
+                    _('Directory "%s" is not writable') % dirname,
                     _('You do not have permissions to create files '
                       'in this directory.'))
                 return
@@ -503,9 +512,9 @@ class FileTransfersWindow:
                        file_name=file_props.name)
 
     def set_status(self, file_props, status):
-        """
+        '''
         Change the status of a transfer to state 'status'
-        """
+        '''
         iter_ = self.get_iter_by_sid(file_props.type_, file_props.sid)
         if iter_ is None:
             return
@@ -550,10 +559,10 @@ class FileTransfersWindow:
         self._select_func(path)
 
     def _format_percent(self, percent):
-        """
+        '''
         Add extra spaces from both sides of the percent, so that progress
         string has always a fixed size
-        """
+        '''
         _str = '          '
         if percent != 100.:
             _str += ' '
@@ -562,27 +571,27 @@ class FileTransfersWindow:
         _str += str(percent) + '%          \n'
         return _str
 
-    def _get_eta_and_speed(self, full_size, transfered_size, file_props):
-        if not file_props.transfered_size:
+    def _get_eta_and_speed(self, full_size, transferred_size, file_props):
+        if not file_props.transferred_size:
             return 0., 0.
 
         if file_props.elapsed_time == 0:
             return 0., 0.
 
-        if len(file_props.transfered_size) == 1:
-            speed = round(float(transfered_size) / file_props.elapsed_time)
+        if len(file_props.transferred_size) == 1:
+            speed = round(float(transferred_size) / file_props.elapsed_time)
         else:
-            # first and last are (time, transfered_size)
-            first = file_props.transfered_size[0]
-            last = file_props.transfered_size[-1]
-            transfered = last[1] - first[1]
+            # first and last are (time, transferred_size)
+            first = file_props.transferred_size[0]
+            last = file_props.transferred_size[-1]
+            transferred = last[1] - first[1]
             tim = last[0] - first[0]
             if tim == 0:
                 return 0., 0.
-            speed = round(float(transfered) / tim)
+            speed = round(float(transferred) / tim)
         if speed == 0.:
             return 0., 0.
-        remaining_size = full_size - transfered_size
+        remaining_size = full_size - transferred_size
         eta = remaining_size / speed
         return eta, speed
 
@@ -593,7 +602,7 @@ class FileTransfersWindow:
         if file_props.tt_account:
             # file transfer is set
             account = file_props.tt_account
-            if account in app.connections:
+            if account in app.settings.get_active_accounts():
                 # there is a connection to the account
                 client = app.get_client(account)
                 client.get_module('Bytestream').remove_transfer(file_props)
@@ -607,10 +616,10 @@ class FileTransfersWindow:
             event.file_props.sid,
             event.file_props.received_len)
 
-    def set_progress(self, typ, sid, transfered_size, iter_=None):
-        """
-        Change the progress of a transfer with new transfered size
-        """
+    def set_progress(self, typ, sid, transferred_size, iter_=None):
+        '''
+        Change the progress of a transfer with new transferred size
+        '''
         if time.time() - self._last_progress_update < 0.5:
             # Update window every 500ms only
             return
@@ -624,7 +633,7 @@ class FileTransfersWindow:
         if full_size == 0:
             percent = 0
         else:
-            percent = round(float(transfered_size) / full_size * 100, 1)
+            percent = round(float(transferred_size) / full_size * 100, 1)
         if iter_ is None:
             iter_ = self.get_iter_by_sid(typ, sid)
         if iter_ is not None:
@@ -632,24 +641,24 @@ class FileTransfersWindow:
             if self.model[iter_][Column.PERCENT] == 0 and int(percent > 0):
                 just_began = True
             text = self._format_percent(percent)
-            if transfered_size == 0:
+            if transferred_size == 0:
                 text += '0'
             else:
-                text += GLib.format_size_full(transfered_size, self.units)
+                text += GLib.format_size_full(transferred_size, self.units)
             text += '/' + GLib.format_size_full(full_size, self.units)
             # Kb/s
 
             # remaining time
             if file_props.offset:
-                transfered_size -= file_props.offset
+                transferred_size -= file_props.offset
                 full_size -= file_props.offset
 
             if file_props.elapsed_time > 0:
-                file_props.transfered_size.append((file_props.last_time,
-                                                   transfered_size))
-            if len(file_props.transfered_size) > 6:
-                file_props.transfered_size.pop(0)
-            eta, speed = self._get_eta_and_speed(full_size, transfered_size,
+                file_props.transferred_size.append((file_props.last_time,
+                                                   transferred_size))
+            if len(file_props.transferred_size) > 6:
+                file_props.transferred_size.pop(0)
+            eta, speed = self._get_eta_and_speed(full_size, transferred_size,
                                                  file_props)
 
             self.model.set(iter_, Column.PROGRESS, text)
@@ -675,7 +684,7 @@ class FileTransfersWindow:
             if file_props.connected is False:
                 status = 'stop'
             self.model.set(iter_, 0, self.icons[status])
-            if transfered_size == full_size:
+            if transferred_size == full_size:
                 # If we are receiver and this is a jingle session
                 if (file_props.type_ == 'r' and
                         file_props.session_type == 'jingle' and
@@ -689,10 +698,10 @@ class FileTransfersWindow:
                 self._select_func(path)
 
     def get_iter_by_sid(self, typ, sid):
-        """
+        '''
         Return iter to the row, which holds file transfer, identified by the
         session id
-        """
+        '''
         iter_ = self.model.get_iter_first()
         while iter_:
             if typ + sid == self.model[iter_][Column.SID]:
@@ -702,7 +711,7 @@ class FileTransfersWindow:
     @staticmethod
     def __convert_date(epoch: float) -> str:
         # Converts date-time from seconds from epoch to iso 8601
-        dt = datetime.utcfromtimestamp(epoch)
+        dt = datetime.fromtimestamp(epoch, timezone.utc)
         return dt.isoformat() + 'Z'
 
     def get_send_file_props(self,
@@ -712,10 +721,10 @@ class FileTransfersWindow:
                             file_name: str,
                             file_desc: str = ''
                             ) -> Optional[FileProp]:
-        """
+        '''
         Create new file_props object and set initial file transfer
         properties in it
-        """
+        '''
         if os.path.isfile(file_path):
             stat = os.stat(file_path)
         else:
@@ -747,9 +756,9 @@ class FileTransfersWindow:
         return file_props
 
     def add_transfer(self, account, contact, file_props):
-        """
+        '''
         Add new transfer to FT window and show the FT window
-        """
+        '''
         if file_props is None:
             return
 
@@ -791,19 +800,19 @@ class FileTransfersWindow:
         self._on_open_folder_menuitem_activate(widget)
 
     def _set_cleanup_sensitivity(self):
-        """
+        '''
         Check if there are transfer rows and set cleanup_button sensitive, or
         insensitive if model is empty
-        """
+        '''
         if not self.model:
             self._ui.cleanup_button.set_sensitive(False)
         else:
             self._ui.cleanup_button.set_sensitive(True)
 
     def _set_all_insensitive(self):
-        """
+        '''
         Make all buttons/menuitems insensitive
-        """
+        '''
         self._ui.pause_resume_button.set_sensitive(False)
         self._ui.pause_resume_menuitem.set_sensitive(False)
         self._ui.remove_menuitem.set_sensitive(False)
@@ -813,10 +822,10 @@ class FileTransfersWindow:
         self._set_cleanup_sensitivity()
 
     def _set_buttons_sensitive(self, path, is_row_selected):
-        """
+        '''
         Make buttons/menuitems sensitive as appropriate to the state of file
         transfer located at path 'path'
-        """
+        '''
         if path is None:
             self._set_all_insensitive()
             return
@@ -853,9 +862,9 @@ class FileTransfersWindow:
         return True
 
     def _selection_changed(self, args):
-        """
+        '''
         Selection has changed - change the sensitivity of the buttons/menuitems
-        """
+        '''
         selection = args
         selected = selection.get_selected_rows()
         if selected[1] != []:
@@ -914,7 +923,7 @@ class FileTransfersWindow:
             file_props.paused = True
             self.set_status(file_props, 'pause')
             # Reset that to compute speed only when we resume
-            file_props.transfered_size = []
+            file_props.transferred_size = []
             self._toggle_pause_continue(False)
 
     def _on_cancel_button_clicked(self, widget):
@@ -929,7 +938,7 @@ class FileTransfersWindow:
     def cancel_transfer(self, file_props: FileProp) -> None:
         # TODO: does not cancel transfer somehow
         account = file_props.tt_account
-        if account is None or account not in app.connections:
+        if account is None or account not in app.settings.get_active_accounts():
             return
         client = app.get_client(account)
         # Check if we are in a IBB transfer
@@ -966,9 +975,9 @@ class FileTransfersWindow:
             event.time)
 
     def _on_transfers_list_key_press_event(self, widget, event):
-        """
+        '''
         When a key is pressed in the treeviews
-        """
+        '''
         iter_ = None
         try:
             iter_ = self._ui.transfers_list.get_selection().get_selected()[1]
